@@ -77,39 +77,84 @@ def _open_file(filename_or_file_object, mode = 'rb'):
         filehandle = filename_or_file_object
     return filehandle
 
+import cStringIO
+import codecs
+class UnicodeWriter:
+    """
+    A CSV writer which will write rows to CSV file "f",
+    which is encoded in the given encoding.
+    """
+
+    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
+        # Redirect output to a queue
+        self.queue = cStringIO.StringIO()
+        self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
+        self.stream = f
+        self.encoder = codecs.getincrementalencoder(encoding)()
+
+    def writerow(self, row):
+        self.writer.writerow([unicode(s).encode("utf-8") for s in row])
+        # Fetch UTF-8 output from the queue ...
+        data = self.queue.getvalue()
+        data = data.decode("utf-8")
+        # ... and reencode it into the target encoding
+        data = self.encoder.encode(data)
+        # write to the target stream
+        self.stream.write(data)
+        # empty queue
+        self.queue.truncate(0)
+
+    def writerows(self, rows):
+        for row in rows:
+            self.writerow(row)
 
 class DF2CSV:
     """A converter for comma-seperated-value files and DataFrames."""
 
-    def read(self, filename_or_file_object, header=True, dialect=None, skip_rows = 0, columns_to_include = None, columns_to_execlude = None, na_text = "NA", type_hints = None, handle_quotes = False, has_row_names = False, r_skipped_first_column_name = False, comment_character = None):
+    def read(self, filename_or_file_object, header=True, dialect=None, 
+        skip_rows = 0, columns_to_include = None, columns_to_exclude = None, 
+        na_text = "NA", type_hints = None, handle_quotes = False, 
+        has_row_names = False, r_skipped_first_column_name = False, 
+        comment_character = None):
         """Read a csv file and turn it into a DataFrame.
-
-        dialect = a dialect (object) valid for csv.reader
-        skipRows = skip n rows at the start of the file
-        columnsToInclude = column names if you want just a few
-        columnsToExeclude = column names if you want to exclude some
-        naText = entries matching will be converted to NaN
-        typeHints = {columnName: numpy.dtype} if you want to enforce
-            some types a priori.
-        handleQuotes = if your csv is quoted (outside of the header)
-                if it isn't it can be read faster.
-        r_skipped_first_column_name = if you export from R with write.table(row.names=TRUE),
-            r skips the first column name. This handles this.
+        
+        ::
+        
+            dialect = a dialect (object) valid for csv.reader
+            skipRows = skip n rows at the start of the file
+            columnsToInclude = column names if you want just a few
+            columnsToExeclude = column names if you want to exclude some
+            naText = entries matching will be converted to NaN
+            typeHints = {columnName: numpy.dtype} if you want to enforce
+                some types a priori.
+            handleQuotes = if your csv is quoted (outside of the header)
+                    if it isn't it can be read faster.
+            r_skipped_first_column_name = if you export from R with write.table(row.names=TRUE),
+                r skips the first column name. This handles this.
 
         """
+        if isinstance(header, csv.Dialect):
+            raise ValueError("You passed a cvs.Dialect as second parameter, but that needs to be a bool. Pass your dialect as third (or dialect=) parameter.")
         try:
             filehandle = _open_file(filename_or_file_object,'rU') 
             return self.__encapsulated_read(filehandle,
-                                           header, dialect, skip_rows, columns_to_include, columns_to_execlude, na_text, type_hints, handle_quotes, has_row_names, r_skipped_first_column_name, comment_character)
+                                           header, dialect, skip_rows, columns_to_include, 
+                                           columns_to_exclude, na_text, type_hints, handle_quotes, 
+                                           has_row_names, r_skipped_first_column_name, comment_character)
         except (zlib.error, IOError), e:
             if isinstance(e, zlib.error) or str(e).find('CRC check failed') != -1:
                 filehandle = _open_file(filename_or_file_object,'r') 
                 return self.__encapsulated_read(filehandle,
-                                               header, dialect, skip_rows, columns_to_include, columns_to_execlude, na_text, type_hints, handle_quotes, has_row_names, r_skipped_first_column_name, comment_character)
+                                               header, dialect, skip_rows, columns_to_include, 
+                                               columns_to_exclude, na_text, type_hints, handle_quotes, 
+                                               has_row_names, r_skipped_first_column_name, comment_character)
             else:
                 raise e
 
-    def __encapsulated_read(self, filehandle, header=True, dialect=None, skip_rows = 0, columns_to_include = None, columns_to_execlude = None, na_text = "NA", type_hints = None, handle_quotes = False, has_row_names = False, r_skipped_first_column_name = False, comment_character = None):
+    def __encapsulated_read(self, filehandle, header=True, dialect=None, skip_rows = 0, 
+        columns_to_include = None, columns_to_exclude = None, na_text = "NA", type_hints = None, 
+        handle_quotes = False, has_row_names = False, r_skipped_first_column_name = False, 
+        comment_character = None):
         if dialect is None:
             dialect = ExcelDialect()
         if isinstance(dialect, str):
@@ -155,6 +200,8 @@ class DF2CSV:
                 if r_skipped_first_column_name:
                     should += 1
                 raise ValueError("Wrong number of headers. Should be %i, was %i (potentially including skipped first column)" % (should, len(fields)))
+            if len(fields) != len(set(fields)):
+                raise ValueError("Some fields occur twice")
         else:
             num_fields = len(first_line)
             num_chars = int(math.ceil(math.log10(num_fields)))
@@ -186,7 +233,10 @@ class DF2CSV:
         for i in range(len(fields)):
             field = fields[i]
             try:
-                if (columns_to_include is None or field in columns_to_include) and (columns_to_execlude is None or (not field in columns_to_execlude)):
+                if (columns_to_include is None or field in columns_to_include) and (columns_to_exclude is None or (not field in columns_to_exclude)):
+                    if columns_to_include is not None and field in columns_to_include:
+                        # Only use the first instance of an included column
+                        columns_to_include.remove(field)
                     fields_used.append(field)
                     if field in type_hints:
                         try:
@@ -222,10 +272,11 @@ class DF2CSV:
     
     def write(self, data_frame, filename_or_file_object, dialect='excel'):
         """Write a DataFrame to a comma seperated value file."""
-        filehandle = _open_file(filename_or_file_object, 'wb')
+        filehandle = _open_file(filename_or_file_object, 'wu')
         if dialect == 'excel':
             dialect = ExcelDialect()
-        writer = csv.writer(filehandle, dialect=dialect)
+        #writer = csv.writer(filehandle, dialect=dialect)
+        writer = UnicodeWriter(filehandle, dialect=dialect)
         if not data_frame.row_names is None:
             header = ["Row"]
             header.extend(data_frame.columns_ordered)
@@ -263,6 +314,19 @@ class DF2CSV:
                     columns[i].append(na_text)
         return columns
 
+class DF2TSV(DF2CSV):
+    """A L{DF2CSV()} with dialect permanently set to TabDialect"""
+
+    def read(self, filename_or_file_object, **kwargs):
+        kwargs['dialect'] = TabDialect()
+        return DF2CSV.read(self,filename_or_file_object, **kwargs)
+
+    def write(self, data_frame, filename_or_file_object, **kwargs):
+        kwargs['dialect'] = TabDialect()
+        return DF2CSV.write(self, data_frame, filename_or_file_object, **kwargs)
+
+
+
 
 class DF2ARFF:
     """ARFF file format (Weka) to DataFrame converter"""
