@@ -98,6 +98,14 @@ class DataFrameTests(unittest.TestCase):
         for x, y in zip(c.get_column_view('Field2').mask, [False, False, False, False, True, False]):
             self.assertEqual(x,y)
 
+    def testRBindSuceedsWithMaskedUnicode(self):
+        a = DataFrame( { "Field1": [1, 2,3], "Field2": [u'abc','def','hgi']}, ['Field1','Field2'])
+        b = DataFrame( { "Field1": [1, 2,3], "Field2": [u'abc',None,'hgi']}, ['Field1','Field2'])
+        c = a.rbind_copy(b)
+        self.assertTrue(isinstance(c.get_value(2,'Field2'), unicode))
+        for x, y in zip(c.get_column_view('Field2').mask, [False, False, False, False, True, False]):
+            self.assertEqual(x,y)
+
 
     def insert_column_raises_on_duplicate_column(self):
         a = DataFrame( { "Field1": [1, 2,3], "Field2": ['abc','def','hgi']}, ['Field1','Field2'])
@@ -1443,6 +1451,14 @@ class DataFramePicklingTests(unittest.TestCase):
         b = cPickle.load(op)
         self.assertEqual(a, b)
 
+    def testEmptyPickle(self):
+        a = DataFrame ( {"one": [] })
+        self.assertEqual(a.get_column('one').dtype, numpy.dtype('object'))
+        op = StringIO.StringIO()
+        cPickle.dump(a, op)
+        op.seek(0,0)
+        b = cPickle.load(op)
+        self.assertEqual(a, b)
 
     def test_mask_dumping(self):
         a = DataFrame ( {
